%ProjectionProgram SCRIPT - MAIN SCRIPT FOR NUMERICAL TESTS -
%PHD Jorge Barrios Ginart - Universidade Federal de Gois 2014
%Version 07/11/2014 -This is the main file for the construction of the
%performance profiles from projection onto simplicial cones by a
%semi-smooth Newton method and Picards methods on np generated random
%problems of dimension m.
%References
%[1] O.P.Ferreira, S.Z. Nmeth. Projection onto simplicial cones by a
%semi-smooth Newton method. Optimization Letters. 
%[2] O.P.Ferreira, S.Z. Nmeth. Projection onto simplicial cones by
%Picards method. Preprint.
%[3] E.D.Dolan, J.J.Mor. Brenchmarking optimization software with
%performance profiles. Math. program.,Ser.A 91:201-213(2002).

% Clear Workspace and Command Window.
clear all
clc

%Save text of MATLAB session.
diary on

%Methods combinations 1-NPP2 
mCombination = 1;

% Initialize method parameters.
%///Parameters to change for each type of problem.
np   = 10%1000;            %number of problems.
n    = 10%1000;            %size of the problem.
tola  = -7;             %tolerance exponent a.
tolb  = -13;            %tolerance exponent b for solve problems from tol=10^tola to 10^tolb.
bMat = 1000000;         %bound for random values.
mIt  = 1000000;         %maximum accepted number of iterations.
coef = 1;               %coeficient for N y P =1 (Max value for P2 =850). 
repit= 10;              %number of executions for each problem (fixed.)
%///Fixed Parameters
cN   = 5000;            %maximum accepted condition number for A.
e1   = 1.0e-15;         %auxiliary perturbation 1 used in the calculation of b.
e2   = 1.0e-16;         %auxiliary perturbation 2 used in the calculation of bC.
upb  = 1/3;             %upper bound for b defined in [1,Th 2,Eq 10].
lob  = 0;               %lower bound for b defined in [1,Th 2,Eq 10].
lobbC = 0;              %lower bound for b defined in [1,Th 2,Eq 10].

rM   = 1.0e13;          %upper bound for performance radio defined in [3].
I    = eye(n);          %N-by-N identity matrix.

for i=1:np   
sprintf('Problem %d of %d',i,np) 

%Generate the matrix A with accepted condition number  
cD = Inf;
A =zeros(n);
while cD>cN
b    = (lob+e1)+(upb-lob-2*e1)*rand;    %parameter b randomly fixed in (lob,upb) defined in [1,Th 2,Eq 10].
bC   = (lobbC+e2)+(b-lobbC-2*e2)*rand;  %parameter bC randomly fixed in (lobbC,b) defined to construct the matrix A.
A    = -bMat + (2*bMat).*rand(n);       %initial random matrix. 
[S V D] = svd(A);                       %SVD of the random matrix.
A    = S*(sqrt(coef*I+bC*(V/V(1,1))))*D;  %recalculating A, modifying the matrix V,
                                          %such that the condition in [1,Th2,Eq 10] is fulfilled.

%A  = -bMat + (2*bMat).*rand(n,n); 
% A=getA(n);
% norm(A'*A-I)
cD=cond(A);
end      

%Computing z from A and the generated solution u and the initial point x0.
u  = -bMat + (2*bMat).*rand(n,1);          %generate the solution u.
z  = A'\(((A'*A-I)*max(u,0))+u);           %equivalent expression (inv(A')*(((A'*A-I)*max(u,0))+u))
x0 = -bMat + (2*bMat).*rand(n,1);          %generate the initial point x0.
setProb(A,x0,z,u,i);
end

for exp=tola:-3:tolb
    tol=10^exp; 
    stats = zeros(np,3);
    stats2 = zeros(np,3);
    timeN= zeros(repit,1);
    timeP= zeros(repit,1);
    timeP2= zeros(repit,1);
    tpN  = zeros(np,1);
    rpN  = zeros(np,1);
    ipN  = zeros(np,1);
    tpP  = zeros(np,1);
    rpP  = zeros(np,1);
    ipP  = zeros(np,1);
    tpP2  = zeros(np,1);
    rpP2  = zeros(np,1);
    ipP2  = zeros(np,1);
    
    
    for i=1:np   
    %sprintf('Problem %d of %d',i,np)
            [A x0 z u] = getProb(i,n);
            disp('N-P-P2')
            for j=1:repit    
            [ipN(i) timeN(j) erN uN]=ssNewton(n,A,x0,z,tol,mIt,u);
            %sprintf('Done Newton')
            [ipP(i) timeP(j) erP uP]=Picard(n,A,x0,z,tol,mIt,u);
            %sprintf('Done Picard')
            [ipP2(i) timeP2(j) erP2 uP2]=Picard2(n,A,x0,z,tol,mIt,u);
            %sprintf('Done Picard2')
            end

            tpN(i)=median(timeN);
            tpP(i)=median(timeP);
            tpP2(i)=median(timeP2);

            Aux=[tpN(i); tpP(i); tpP2(i)];
            tBest=min(Aux);

            if tBest<Inf
                pos=find(Aux<=tBest+0.01*tBest);
                stats(i,pos)=1;
            else
                stats(i,:)=0;
                Aux(pos)=tBest;
            end

            tpN(i)=Aux(1);
            tpP(i)=Aux(2);
            tpP2(i)=Aux(3); 

            pos2=find(Aux==Inf);
            stats2(i,pos2)=1;    

            if erN>tol;  rpN(i)=rM;  else rpN(i)=tpN(i)/tBest;   end
            if erP>tol;  rpP(i)=rM;  else rpP(i)=tpP(i)/tBest;   end
            if erP2>tol; rpP2(i)=rM; else rpP2(i)=tpP2(i)/tBest; end

       end %for i=1:np 
    
            %Ploting Performance Profile from [3]
            disp('N-P-P2')
            tau=linspace(1,12,np); 
            [fil,siz]=size(tau);
            rhoN=zeros(siz,1);
            rhoP=zeros(siz,1);
            rhoP2=zeros(siz,1);
            for q=1:siz
            rhoN(q)=sum(rpN<=tau(q))/np;
            rhoP(q)=sum(rpP<=tau(q))/np;
            rhoP2(q)=sum(rpP2<=tau(q))/np;
            end 

            h=figure;
            plot(tau,rhoN,'r',tau,rhoP,'b-.',tau,rhoP2,'k:','LineWidth',3)
            axis([1 2 0 1.05])
            title(strcat('tol=',num2str(tol)))
            legend('ssNewton','Picard','Picard2','Location','Best')
            legend boxoff 
            saveas(h,strcat(num2str(n),'x2_',num2str(exp)),'png')
            saveas(h,strcat(num2str(n),'x2_',num2str(exp)),'jpg')
            saveas(h,strcat(num2str(n),'x2_',num2str(exp)),'eps')
            saveas(h,strcat(num2str(n),'x2_',num2str(exp)),'bmp')
            close(h);

            h=figure;
            plot(tau,rhoN,'r',tau,rhoP,'b-.',tau,rhoP2,'k:','LineWidth',3)
            axis([1 3 0 1.05])
            title(strcat('tol=',num2str(tol)))
            legend('ssNewton','Picard','Picard2','Location','Best')
            legend boxoff 
            saveas(h,strcat(num2str(n),'x3_',num2str(exp)),'png')
            saveas(h,strcat(num2str(n),'x3_',num2str(exp)),'jpg')
            saveas(h,strcat(num2str(n),'x3_',num2str(exp)),'eps')
            saveas(h,strcat(num2str(n),'x3_',num2str(exp)),'bmp')
            close(h);

            h=figure;
            plot(tau,rhoN,'r',tau,rhoP,'b-.',tau,rhoP2,'k:','LineWidth',3)
            axis([1 4 0 1.05])
            title(strcat('tol=',num2str(tol)))
            legend('ssNewton','Picard','Picard2','Location','Best')
            legend boxoff 
            saveas(h,strcat(num2str(n),'x4_',num2str(exp)),'png')
            saveas(h,strcat(num2str(n),'x4_',num2str(exp)),'jpg')
            saveas(h,strcat(num2str(n),'x4_',num2str(exp)),'eps')
            saveas(h,strcat(num2str(n),'x4_',num2str(exp)),'bmp')
            close(h);

            h=figure;
            plot(tau,rhoN,'r',tau,rhoP,'b-.',tau,rhoP2,'k:','LineWidth',3)
            axis([1 5 0 1.05])  
            title(strcat('tol=',num2str(tol)))
            legend('ssNewton','Picard','Picard2','Location','Best')
            legend boxoff 
            saveas(h,strcat(num2str(n),'x5_',num2str(exp)),'png')
            saveas(h,strcat(num2str(n),'x5_',num2str(exp)),'jpg')
            saveas(h,strcat(num2str(n),'x5_',num2str(exp)),'eps')
            saveas(h,strcat(num2str(n),'x5_',num2str(exp)),'bmp')
            close(h);

            h=figure;
            plot(tau,rhoN,'r',tau,rhoP,'b-.',tau,rhoP2,'k:','LineWidth',3) 
            axis([1 6 0 1.05])  
            title(strcat('tol=',num2str(tol)))
            legend('ssNewton','Picard','Picard2','Location','Best')
            legend boxoff 
            saveas(h,strcat(num2str(n),'x6_',num2str(exp)),'png')
            saveas(h,strcat(num2str(n),'x6_',num2str(exp)),'jpg')
            saveas(h,strcat(num2str(n),'x6_',num2str(exp)),'eps')
            saveas(h,strcat(num2str(n),'x6_',num2str(exp)),'bmp')
            close(h);

            h=figure;
            plot(tau,rhoN,'r',tau,rhoP,'b-.',tau,rhoP2,'k:','LineWidth',3)
            axis([1 7 0 1.05])  
            title(strcat('tol=',num2str(tol)))
            legend('ssNewton','Picard','Picard2','Location','Best')
            legend boxoff 
            saveas(h,strcat(num2str(n),'x7_',num2str(exp)),'png')
            saveas(h,strcat(num2str(n),'x7_',num2str(exp)),'jpg')
            saveas(h,strcat(num2str(n),'x7_',num2str(exp)),'eps')
            saveas(h,strcat(num2str(n),'x7_',num2str(exp)),'bmp')
            close(h);

            h=figure;
            plot(tau,rhoN,'r',tau,rhoP,'b-.',tau,rhoP2,'k:','LineWidth',3)
            axis([1 8 0 1.05])  
            title(strcat('tol=',num2str(tol))) 
            legend('ssNewton','Picard','Picard2','Location','Best')
            legend boxoff 
            saveas(h,strcat(num2str(n),'x8_',num2str(exp)),'png')
            saveas(h,strcat(num2str(n),'x8_',num2str(exp)),'jpg')
            saveas(h,strcat(num2str(n),'x8_',num2str(exp)),'eps')
            saveas(h,strcat(num2str(n),'x8_',num2str(exp)),'bmp')
            close(h);

            h=figure;
            plot(tau,rhoN,'r',tau,rhoP,'b-.',tau,rhoP2,'k:','LineWidth',3)
            axis([1 10 0 1.05])  
            title(strcat('tol=',num2str(tol)))
            legend('ssNewton','Picard','Picard2','Location','Best')
            legend boxoff 
            saveas(h,strcat(num2str(n),'x10_',num2str(exp)),'png')
            saveas(h,strcat(num2str(n),'x10_',num2str(exp)),'jpg')
            saveas(h,strcat(num2str(n),'x10_',num2str(exp)),'eps')
            saveas(h,strcat(num2str(n),'x10_',num2str(exp)),'bmp')
            close(h);

            h=figure;
            plot(tau,rhoN,'r',tau,rhoP,'b-.',tau,rhoP2,'k:','LineWidth',3)
            axis([1 12 0 1.05])  
            title(strcat('tol=',num2str(tol)))
            legend('ssNewton','Picard','Picard2','Location','Best')
            legend boxoff 
            saveas(h,strcat(num2str(n),'x12_',num2str(exp)),'png')
            saveas(h,strcat(num2str(n),'x12_',num2str(exp)),'jpg')
            saveas(h,strcat(num2str(n),'x12_',num2str(exp)),'eps')
            saveas(h,strcat(num2str(n),'x12_',num2str(exp)),'bmp')
            close(h);


            s=sum(stats,1);
            s2=sum(stats2,1); 
            s3=stats2.*[1:np;1:np;1:np]';
            
            
            st=[mean(tpN(find((tpN>0)&(tpN<Inf)))) mean(tpP(find((tpP>0)&(tpP<Inf)))) mean(tpP2(find((tpP2>0)&(tpP2<Inf))))
                mean(ipN(find((ipN>0)&(ipN<Inf)))) mean(ipP(find((ipP>0)&(ipP<Inf)))) mean(ipP2(find((ipP2>0)&(ipP2<Inf))))];
            st(find(isnan(st)==1))=Inf;
            
            sprintf('Unresolved(N-P-P2):')
            disp(s3)
            
            sprintf('Number of problems: %d, Tolerance:%0.5g, Size of the problem: %d\nEfficiency(N-P-P2): %d  %d  %d\nUnresolved(N-P-P2): %d  %d  %d\nMean  Time(N-P-P2): %d  %d  %d\nTotal Time(N-P-P2): %d  %d  %d\nMean  Iter(N-P-P2): %d  %d  %d\nTotal Iter(N-P-P2): %d  %d  %d'...
            ,np,tol,n,...
            s(1),s(2),s(3),...
            s2(1),s2(2),s2(3),...
            st(1,1),st(1,2),st(1,3),...
            st(1,1)*np,st(1,2)*np,st(1,3)*np,...
            st(2,1),st(2,2),st(2,3),...
            st(2,1)*np,st(2,2)*np,st(2,3)*np)
        
            
            clear tol stats stats2 timeN timeP timeP2 tpN rpN ipN tpP rpP ipP tpP2 rpP2 ipP2 s3;
end %for exp=tola:-1:tolb



if input('Do you want to delete the problems yes 1 not 0 \n')
    delete('*.bin')
end
if input('Do you want to delete the figures png yes 1 not 0 \n')
    delete('*.png')
end
if input('Do you want to delete the figures jpg yes 1 not 0 \n')
    delete('*.jpg')
end
if input('Do you want to delete the figures eps yes 1 not 0 \n')
    delete('*.eps')
end
if input('Do you want to delete the figures bmp yes 1 not 0 \n')
    delete('*.bmp')
end

clear all
diary off
